package handler

import (
	"crypto/sha256"
	"encoding/hex"
	"encoding/json"
	"fmt"
	"fusenapi/constants"
	"fusenapi/utils/basic"
	"github.com/zeromicro/go-zero/core/logx"
	"github.com/zeromicro/go-zero/rest/httpx"
	"net/http"
	"time"

	"fusenapi/server/websocket/internal/svc"
	"fusenapi/server/websocket/internal/types"
)

func RenderNotifyHandler(svcCtx *svc.ServiceContext) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		var req types.RenderNotifyReq
		_, err := basic.RequestParse(w, r, svcCtx, &req)
		if err != nil {
			httpx.OkJsonCtx(r.Context(), w, basic.Response{
				Code:    basic.CodeRequestParamsErr.Code,
				Message: "err param",
				Data:    nil,
			})
			return
		}
		if len(req.NotifyList) == 0 {
			httpx.OkJsonCtx(r.Context(), w, basic.Response{
				Code:    basic.CodeRequestParamsErr.Code,
				Message: "invalid param,notify list is empty",
				Data:    nil,
			})
			return
		}
		if time.Now().Unix()-120 > req.Time {
			httpx.OkJsonCtx(r.Context(), w, basic.Response{
				Code:    basic.CodeRequestParamsErr.Code,
				Message: "invalid param,time is expired",
				Data:    nil,
			})
			return
		}
		//验证签名 sha256
		notifyByte, _ := json.Marshal(req.NotifyList)
		h := sha256.New()
		h.Write([]byte(fmt.Sprintf(constants.RENDER_NOTIFY_SIGN_KEY, string(notifyByte), req.Time)))
		signHex := h.Sum(nil)
		sign := hex.EncodeToString(signHex)
		if req.Sign != sign {
			httpx.OkJsonCtx(r.Context(), w, basic.Response{
				Code:    basic.CodeRequestParamsErr.Code,
				Message: "invalid sign",
				Data:    nil,
			})
			return
		}
		//遍历链接
		mapConnPool.Range(func(key, value any) bool {
			ws, ok := value.(wsConnectItem)
			if !ok {
				return false
			}
			setOutRenderImage(req, ws)
			return true
		})
		httpx.OkJsonCtx(r.Context(), w, basic.Response{
			Code:    200,
			Message: "success",
			Data:    nil,
		})
	}
}

// 把渲染好的数据放入outchan
func setOutRenderImage(req types.RenderNotifyReq, ws wsConnectItem) {
	ws.mutex.Lock()
	defer ws.mutex.Unlock()
	for _, notifyItem := range req.NotifyList {
		renderKey := ws.getRenderImageMapKey(notifyItem.ProductId, notifyItem.SizeId, notifyItem.TemplateId)
		//查询
		_, ok := ws.renderImage[renderKey]
		if !ok {
			continue
		}
		responseData := types.RenderImageRspMsg{
			ProductId:  notifyItem.ProductId,
			SizeId:     notifyItem.SizeId,
			TemplateId: notifyItem.TemplateId,
			Source:     "我是渲染资源",
		}
		b, _ := json.Marshal(responseData)
		select {
		case <-ws.closeChan:
			return
		case ws.outChan <- b:
			logx.Info("notify send render result to out chan")
		}
		//删掉已经处理的渲染任务
		delete(ws.renderImage, renderKey)
	}
}
